@@ -1,14 +1,14 @@
 """Module with Keras Sequences."""
 from .node2vec_sequence import Node2VecSequence
 from .edge_prediction_sequence import EdgePredictionSequence
-from .edge_prediction_degree_sequence import EdgePredictionDegreeSequence
 from .word2vec_sequence import Word2VecSequence
-from .node_label_neighbours_sequence import NodeLabelNeighboursSequence
+from .node_label_prediction_sequence import NodeLabelPredictionSequence
+from .glove_sequence import GloveSequence
 
 __all__ = [
     "Node2VecSequence",
     "EdgePredictionSequence",
-    "EdgePredictionDegreeSequence",
     "Word2VecSequence",
-    "NodeLabelNeighboursSequence"
+    "NodeLabelPredictionSequence",
+    "GloveSequence"
 ]
